# -*- coding: utf-8 -*-
from setuphelpers import *
import json
import time

# {731F6BAA-A986-45A4-8936-7C3AAAAA760B}      Teams Machine-Wide Installer        1.3.0.12058         MsiExec.exe /I{731F6BAA-A986-45A4-8936-7C3AAAAA760B}


# Installation procedure: https://docs.microsoft.com/fr-fr/microsoftteams/teams-for-vdi#install-or-update-the-teams-desktop-app-on-vdi

# Defining variables
silent_args = 'OPTIONS="noAutoStart=true" ALLUSER=1'  # OPTIONS="noAutoStart=true" ALLUSER=1 ALLUSERS=1
# silent_args = 'ALLUSER=1' #OPTIONS="noAutoStart=true" ALLUSER=1 ALLUSERS=1
app_name = "Microsoft Teams"
app_name_short = "Teams"
app_dir = makepath(programfiles32, "Microsoft", "Teams")
app_path = makepath(app_dir, "current", "Teams.exe")
# sys_json_setup = makepath(app_dir,'setup.json')
# json_setup_content =  json.loads('{"noAutoStart":true,"--exeName":"Teams.exe"}')


def install():
    # Initializing variables
    package_version = control.version.split("-")[0]
    bin_name = glob.glob("Teams_windows*.msi")[0]

    # Uninstalling older versions if found
    for uninstall in installed_softwares(name=app_name_short):
        if Version(uninstall["version"]) < Version(package_version):
            print("Older %s version found (%s)" % (app_name, uninstall["version"]))
            print("Removing: %s (%s)" % (uninstall["name"], uninstall["version"]))
            run_notfatal(uninstall_cmd(uninstall["key"]))
            time.sleep(15)

    # Adding fake WVD/VDI RegKey
    # Un/comment if needed
    registry_set(HKEY_LOCAL_MACHINE, r"SOFTWARE\Microsoft\Teams", "IsWVDEnvironment", 1, type=REG_DWORD)
    registry_set(HKEY_LOCAL_MACHINE, r"SOFTWARE\VMware, Inc.\VMware VDM\Agent", "", "")
    registry_set(HKEY_LOCAL_MACHINE, r"SOFTWARE\Citrix\PortICA", "", "")

    # Installing the package system wide
    print("Installing: %s" % bin_name)
    install_msi_if_needed(bin_name, properties=silent_args, min_version=package_version, accept_returncodes=[0, 3010, 1605, 1614, 1641], timeout=900)

    # Disabling AutoStart
    if iswin64():
        registry_delete(HKEY_LOCAL_MACHINE, r"SOFTWARE\WOW6432Node\Microsoft\Windows\CurrentVersion\Run", "Teams")
    registry_delete(HKEY_LOCAL_MACHINE, r"SOFTWARE\Microsoft\Windows\CurrentVersion\Run", "Teams")

    # Removing shortcuts
    remove_desktop_shortcut(app_name)

    # Adding Firewall Rules for Teams
    run_powershell(f'Remove-NetFirewallRule -DisplayName "{app_name}" -ErrorAction SilentlyContinue', accept_returncodes=[0, 1, 3010])
    run_powershell(
        f'New-NetFirewallRule -DisplayName "{app_name}" -Group "{app_name}" -Direction Inbound -Action Allow -Program "{app_path}" -Enabled True '
    )

    # Cleaning fake WVD/VDI RegKey properly
    # Un/comment if needed
    if reg_value_exists(HKEY_LOCAL_MACHINE, r"SOFTWARE\Microsoft\Teams", "IsWVDEnvironment"):
        registry_delete(HKEY_LOCAL_MACHINE, r"SOFTWARE\Microsoft\Teams", "IsWVDEnvironment")
    if reg_key_exists(HKEY_LOCAL_MACHINE, r"SOFTWARE\VMware, Inc."):
        registry_deletekey(HKEY_LOCAL_MACHINE, r"SOFTWARE\VMware, Inc.\VMware VDM\Agent", "", force=False)
        registry_deletekey(HKEY_LOCAL_MACHINE, r"SOFTWARE\VMware, Inc.\VMware VDM", "", force=False)
        registry_deletekey(HKEY_LOCAL_MACHINE, r"SOFTWARE\VMware, Inc.", "", force=False)
    if reg_key_exists(HKEY_LOCAL_MACHINE, r"SOFTWARE\Citrix\PortICA"):
        registry_deletekey(HKEY_LOCAL_MACHINE, r"SOFTWARE\Citrix\PortICA", "", force=False)


def uninstall():
    # Uninstalling the software
    for to_uninstall in installed_softwares(name=app_name_short):
        print("Removing: %s (%s)" % (to_uninstall["name"], to_uninstall["version"]))
        killalltasks(control.get_impacted_process_list())
        run(uninstall_cmd(to_uninstall["key"]))
        wait_uninstallkey_absent(to_uninstall["key"])

    # Removing Firewall Rules for Teams
    run_powershell(f'Remove-NetFirewallRule -DisplayName "{app_name}" -ErrorAction SilentlyContinue', accept_returncodes=[0, 1, 3010])

    # Removing remaining files
    if isdir(app_dir):
        killalltasks(control.get_impacted_process_list())
        remove_tree(app_dir)


def session_setup():
    # Initializing variables
    user_conf_dir = makepath(application_data, "Microsoft", "Teams")
    json_config = makepath(user_conf_dir, "desktop-config.json")
    json_settings = makepath(user_conf_dir, "settings.json")

    user_app_dir = makepath(user_local_appdata, "Microsoft", "Teams")
    user_app_path = makepath(user_app_dir, "current", "Teams.exe")
    user_app_updater = makepath(user_app_dir, "Update.exe")
    # user_app_version = registry_readstring(HKEY_CURRENT_USER,r'Software\Microsoft\Windows\CurrentVersion\Uninstall\Teams','DisplayVersion')

    user_app_progdata_dir = makepath("C:", "ProgramData", get_current_user(), "Microsoft", "Teams")
    user_app_progdata_path = makepath(user_app_progdata_dir, "current", "Teams.exe")
    user_app_progdata_updater = makepath(user_app_progdata_dir, "Update.exe")

    user_silent_args_uninst = "--uninstall -s"
    processes_to_kill = control.impacted_process.split(",")

    # Uninstalling from user environment (procedure: https://docs.microsoft.com/microsoftteams/msi-deployment)
    if isfile(user_app_path):
        print("Uninstalling Microsoft Teams from user environment")
        killalltasks(processes_to_kill)
        run_notfatal('"%s" %s' % (user_app_updater, user_silent_args_uninst))
    if isdir(user_app_dir):
        remove_tree(user_app_dir)
    if isfile(user_app_progdata_path):
        print("Uninstalling Microsoft Teams from user ProgramData environment")
        killalltasks(processes_to_kill)
        run_notfatal('"%s" %s' % (user_app_progdata_updater, user_silent_args_uninst))
    if isdir(user_app_progdata_dir):
        killalltasks(processes_to_kill)
        remove_tree(user_app_progdata_dir)

    if reg_value_exists(HKEY_CURRENT_USER, r"Software\Microsoft\Office\Teams", "PreventInstallationFromMsi"):
        registry_delete(HKEY_CURRENT_USER, r"Software\Microsoft\Office\Teams", "PreventInstallationFromMsi")

    # No longer Prefill UPN
    registry_set(HKEY_CURRENT_USER, r"Software\Microsoft\Office\Teams", "SkipUpnPrefill", 1, type=REG_DWORD)

    # Define user settings (procedure: https://techcommunity.microsoft.com/t5/microsoft-teams/how-to-disable-check-for-updates-in-microsoft-teams/m-p/54644)
    print("Setting up user settings")
    if not isdir(user_conf_dir):
        mkdirs(user_conf_dir)

    # desktop-config.json
    if isfile(json_config):
        json_config_content = load_json(json_config)
    else:
        json_config_content = json.loads('{"appPreferenceSettings":{}}')
        # json_config_content = json.load({})

    json_config_content["appPreferenceSettings"]["registerAsIMProvider"] = False
    json_config_content["appPreferenceSettings"]["openAsHidden"] = True
    json_config_content["appPreferenceSettings"]["runningOnClose"] = True
    json_config_content["openAtLogin"] = False
    json_config_content["isForeground"] = True
    json_config_content["wamFallbackInProgress"] = False
    json_config_content["disableWarningOnOpenKeyRegistered"] = True
    json_config_content["isAppFirstRun"] = False

    write_json(json_config, json_config_content)

    # settings.json
    if isfile(json_settings):
        json_settings_content = load_json(json_settings)
    else:
        json_settings_content = json.loads('{"baseSettings":{},"initializedSettings":{},"preauthSettings":{"samplingRules":{}}}')
        # json_settings_content = json.load({})

    # Update part
    json_settings_content["baseSettings"]["enableBlockSilentUpdate"] = True
    json_settings_content["baseSettings"]["disableAllUsersAutoUpdate"] = True
    json_settings_content["baseSettings"]["enableUnAuthUpdates"] = False
    # json_settings_content['initializedSettings']['disableAllUsersAutoUpdate'] = True
    # json_settings_content['preauthSettings']['disableAllUsersAutoUpdate'] = True
    # json_settings_content['preauthSettings']['samplingRules']['updateNotificationEnabled'] = False

    # Telemetry part
    json_settings_content["baseSettings"]["enableAdalMacTelemetryV2"] = False
    json_settings_content["baseSettings"]["enableAdalWinTelemetry"] = False
    json_settings_content["baseSettings"]["proplusAdditionalTelemetryEnabled"] = False
    json_settings_content["baseSettings"]["regionInTelemetryEnabled"] = False
    json_settings_content["baseSettings"]["tenantRegionInTelemetryEnabled"] = False
    # json_settings_content['initializedSettings']['tenantRegionInTelemetryEnabled'] = False
    # json_settings_content['preauthSettings']['tenantRegionInTelemetryEnabled'] = False
    # json_settings_content['preauthSettings']['samplingRules']['tenantRegionInTelemetryEnabled'] = False

    # Other part
    json_settings_content["baseSettings"]["enableRegisterProtocolToRoot"] = False
    # json_settings_content['preauthSettings']['registerAddinOnMeetingPolicySettingsSync'] = False

    write_json(json_settings, json_settings_content)


def load_json(json_filename):
    with open(json_filename) as json_file:
        data = json.load(json_file)
    return data


def write_json(json_filename, data):
    with open(json_filename, "w") as outfile:
        json.dump(data, outfile, sort_keys=True, indent=4, default=str)
